import { useEffect, useRef } from 'react';
import { useAsync } from '@react-devui/hooks';
const FOCUS_KEY = ['ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight', 'Home', 'End', 'Enter', 'Space'];
export function DFocusVisible(props) {
    const { children, onFocusVisibleChange } = props;
    const dataRef = useRef({
        focusByKeydown: false,
        focusVisible: false,
    });
    const asyncCapture = useAsync();
    useEffect(() => {
        const [asyncGroup, asyncId] = asyncCapture.createGroup();
        asyncGroup.fromEvent(window, 'keydown').subscribe({
            next: () => {
                dataRef.current.focusByKeydown = true;
                asyncGroup.setTimeout(() => {
                    dataRef.current.focusByKeydown = false;
                }, 20);
            },
        });
        return () => {
            asyncCapture.deleteGroup(asyncId);
        };
    }, [asyncCapture]);
    return children({
        fvOnFocus: () => {
            if (dataRef.current.focusByKeydown) {
                onFocusVisibleChange === null || onFocusVisibleChange === void 0 ? void 0 : onFocusVisibleChange(true);
                dataRef.current.focusVisible = true;
            }
        },
        fvOnBlur: () => {
            onFocusVisibleChange === null || onFocusVisibleChange === void 0 ? void 0 : onFocusVisibleChange(false);
            dataRef.current.focusVisible = false;
        },
        fvOnKeyDown: (e) => {
            if (!dataRef.current.focusVisible && FOCUS_KEY.includes(e.code)) {
                onFocusVisibleChange === null || onFocusVisibleChange === void 0 ? void 0 : onFocusVisibleChange(true);
            }
        },
    });
}
//# sourceMappingURL=FocusVisible.js.map