import { Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import { useEffect, useId, useRef } from 'react';
import ReactDOM from 'react-dom';
import { filter } from 'rxjs';
import { useAsync, useElement } from '@react-devui/hooks';
import { useLayout } from '../../hooks';
export function DPopup(props) {
    const { children, dPopup, dVisible = false, dContainer, dTrigger = 'hover', dDisabled = false, dEscClosable = true, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dUpdatePosition, onVisibleChange, } = props;
    //#region Context
    const { dScrollEl, dResizeEl } = useLayout();
    //#endregion
    const dataRef = useRef({});
    const asyncCapture = useAsync();
    const uniqueId = useId();
    const scrollEl = useElement(dScrollEl);
    const resizeEl = useElement(dResizeEl);
    const handleTrigger = (visible, behavior) => {
        var _a, _b;
        (_b = (_a = dataRef.current).clearTid) === null || _b === void 0 ? void 0 : _b.call(_a);
        const changeVisible = () => {
            if (isUndefined(visible)) {
                onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(!dVisible);
            }
            else if (!Object.is(dVisible, visible)) {
                onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(visible);
            }
        };
        if (behavior) {
            switch (behavior) {
                case 'hover':
                    if (visible) {
                        dataRef.current.clearTid = asyncCapture.setTimeout(() => changeVisible(), dMouseEnterDelay);
                    }
                    else {
                        dataRef.current.clearTid = asyncCapture.setTimeout(() => changeVisible(), dMouseLeaveDelay);
                    }
                    break;
                case 'popup-hover':
                    if (!visible) {
                        dataRef.current.clearTid = asyncCapture.setTimeout(() => changeVisible(), dMouseLeaveDelay);
                    }
                    break;
                default:
                    break;
            }
        }
        else {
            changeVisible();
        }
    };
    useEffect(() => {
        if (!dDisabled && dVisible && dTrigger === 'click') {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(window, 'click', { capture: true }).subscribe({
                next: () => {
                    dataRef.current.clearClickOut = asyncGroup.requestAnimationFrame(() => {
                        handleTrigger(false);
                    });
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    });
    useEffect(() => {
        if (!dDisabled && dVisible && dEscClosable) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup
                .fromEvent(window, 'keydown')
                .pipe(filter((e) => e.code === 'Escape'))
                .subscribe({
                next: () => {
                    handleTrigger(false);
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    });
    useEffect(() => {
        if (!dDisabled && dVisible) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            const triggerEl = document.querySelector(`[data-popup-triggerid="${uniqueId}"]`);
            if (triggerEl) {
                asyncGroup.onResize(triggerEl, () => {
                    dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
                });
            }
            const popupEl = document.querySelector(`[data-popup-popupid="${uniqueId}"]`);
            if (popupEl) {
                asyncGroup.onResize(popupEl, () => {
                    dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
                });
            }
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, dDisabled, dUpdatePosition, dVisible, uniqueId]);
    useEffect(() => {
        if (!dDisabled && dVisible && dContainer) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(dContainer, 'scroll', { passive: true }).subscribe({
                next: () => {
                    dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
                },
            });
            asyncGroup.onResize(dContainer, () => {
                dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, dContainer, dDisabled, dUpdatePosition, dVisible]);
    useEffect(() => {
        if (!dDisabled && dVisible && scrollEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(scrollEl, 'scroll', { passive: true }).subscribe({
                next: () => {
                    dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, dDisabled, dUpdatePosition, dVisible, scrollEl]);
    useEffect(() => {
        if (!dDisabled && dVisible && resizeEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.onResize(resizeEl, () => {
                dUpdatePosition === null || dUpdatePosition === void 0 ? void 0 : dUpdatePosition();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, dDisabled, dUpdatePosition, dVisible, resizeEl]);
    const childProps = { 'data-popup-triggerid': uniqueId };
    if (!dDisabled) {
        switch (dTrigger) {
            case 'hover':
                childProps.pOnMouseEnter = () => {
                    handleTrigger(true, 'hover');
                };
                childProps.pOnMouseLeave = () => {
                    handleTrigger(false, 'hover');
                };
                break;
            case 'click':
                childProps.pOnClick = () => {
                    var _a, _b;
                    (_b = (_a = dataRef.current).clearClickOut) === null || _b === void 0 ? void 0 : _b.call(_a);
                    handleTrigger();
                };
                break;
            default:
                break;
        }
    }
    const child = children(childProps);
    const popupProps = { 'data-popup-popupid': uniqueId };
    if (!dDisabled && dVisible) {
        switch (dTrigger) {
            case 'hover':
                popupProps.pOnMouseEnter = () => {
                    handleTrigger(true, 'popup-hover');
                };
                popupProps.pOnMouseLeave = () => {
                    handleTrigger(false, 'popup-hover');
                };
                break;
            case 'click':
                popupProps.pOnClick = () => {
                    var _a, _b;
                    (_b = (_a = dataRef.current).clearClickOut) === null || _b === void 0 ? void 0 : _b.call(_a);
                };
                break;
            default:
                break;
        }
    }
    const popupNode = dPopup(popupProps);
    return (_jsxs(_Fragment, { children: [child, !dDisabled && dContainer && ReactDOM.createPortal(popupNode, dContainer)] }));
}
//# sourceMappingURL=Popup.js.map