var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useCallback, useEffect, useImperativeHandle, useRef, useState } from 'react';
import { useAsync, useElement, useIsomorphicLayoutEffect } from '@react-devui/hooks';
import { getClassName, toPx } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useLayout } from '../../hooks';
import { registerComponentMate } from '../../utils';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DAffix' });
function Affix(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dContainer, dTop = 0, dZIndex } = _a, restProps = __rest(_a, ["children", "dContainer", "dTop", "dZIndex"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dScrollEl, dResizeEl } = useLayout();
    //#endregion
    //#region Ref
    const affixRef = useRef(null);
    const referenceRef = useRef(null);
    //#endregion
    const asyncCapture = useAsync();
    const isDefaultContainer = isUndefined(dContainer);
    const [sticky, setSticky] = useState(false);
    const scrollEl = useElement(dScrollEl);
    const resizeEl = useElement(dResizeEl);
    const containerEl = useElement(isDefaultContainer
        ? () => {
            let el = document.getElementById(`${dPrefix}affix-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}affix-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a;
                const el = sticky ? referenceRef.current : affixRef.current;
                return (_a = el === null || el === void 0 ? void 0 : el.parentElement) !== null && _a !== void 0 ? _a : null;
            }
            : dContainer);
    const [positionStyle, setPositionStyle] = useState();
    const [referenceStyle, setReferenceStyle] = useState();
    const updatePosition = useCallback(() => {
        const offsetEl = sticky ? referenceRef.current : affixRef.current;
        if (offsetEl) {
            const offsetRect = offsetEl.getBoundingClientRect();
            let containerTop = 0;
            if (!isDefaultContainer && containerEl) {
                containerTop = containerEl.getBoundingClientRect().top;
            }
            const top = isString(dTop) ? toPx(dTop, true) : dTop;
            const sticky = offsetRect.top - containerTop <= top;
            if (sticky) {
                setPositionStyle({
                    width: offsetRect.width,
                    height: offsetRect.height,
                    left: offsetRect.left,
                    top: top + containerTop,
                });
                setReferenceStyle({
                    width: offsetRect.width,
                    height: offsetRect.height,
                });
            }
            setSticky(sticky);
        }
    }, [containerEl, dTop, isDefaultContainer, sticky]);
    useIsomorphicLayoutEffect(() => {
        updatePosition();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    useEffect(() => {
        if (!isDefaultContainer && containerEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(containerEl, 'scroll', { passive: true }).subscribe({
                next: () => {
                    updatePosition();
                },
            });
            asyncGroup.onResize(containerEl, () => {
                updatePosition();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, containerEl, isDefaultContainer, updatePosition]);
    useEffect(() => {
        if (scrollEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(scrollEl, 'scroll', { passive: true }).subscribe({
                next: () => {
                    updatePosition();
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, scrollEl, updatePosition]);
    useEffect(() => {
        if (resizeEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.onResize(resizeEl, () => {
                updatePosition();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, resizeEl, updatePosition]);
    useImperativeHandle(ref, () => ({
        sticky,
        updatePosition,
    }), [sticky, updatePosition]);
    return (_jsxs(_Fragment, { children: [sticky && (_jsx("div", Object.assign({}, restProps, { ref: referenceRef, className: getClassName(restProps.className, `${dPrefix}affix`), style: Object.assign(Object.assign(Object.assign({}, restProps.style), referenceStyle), { visibility: 'hidden' }), "aria-hidden": true }))), _jsx("div", Object.assign({}, restProps, { ref: affixRef, className: getClassName(restProps.className, `${dPrefix}affix`), style: Object.assign(Object.assign({}, restProps.style), (sticky
                    ? Object.assign({ position: 'fixed', zIndex: dZIndex !== null && dZIndex !== void 0 ? dZIndex : `var(--${dPrefix}zindex-sticky)` }, positionStyle) : {})) }, { children: children }))] }));
}
export const DAffix = React.forwardRef(Affix);
//# sourceMappingURL=Affix.js.map