var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { isArray, isUndefined } from 'lodash';
import React, { useCallback, useEffect, useImperativeHandle, useRef, useState } from 'react';
import { useAsync, useElement, useIsomorphicLayoutEffect } from '@react-devui/hooks';
import { getClassName, scrollTo } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useLayout } from '../../hooks';
import { registerComponentMate } from '../../utils';
const DOT_INDICATOR = Symbol();
const LINE_INDICATOR = Symbol();
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DAnchor' });
function Anchor(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dList, dPage, dDistance = 0, dScrollBehavior = 'instant', dIndicator = DOT_INDICATOR, onItemClick } = _a, restProps = __rest(_a, ["dList", "dPage", "dDistance", "dScrollBehavior", "dIndicator", "onItemClick"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dScrollEl, dResizeEl } = useLayout();
    //#endregion
    //#region Ref
    const anchorRef = useRef(null);
    const indicatorRef = useRef(null);
    //#endregion
    const dataRef = useRef({});
    const asyncCapture = useAsync();
    const pageEl = useElement(dPage !== null && dPage !== void 0 ? dPage : dScrollEl);
    const resizeEl = useElement(dResizeEl);
    const [activeHref, setActiveHref] = useState(null);
    const updateAnchor = useCallback(() => {
        if (!pageEl) {
            return;
        }
        const pageTop = pageEl.getBoundingClientRect().top;
        let nearestEl;
        const reduceLinks = (arr) => {
            arr.forEach(({ href, children }) => {
                const el = document.getElementById(href);
                if (el) {
                    const top = el.getBoundingClientRect().top;
                    // Add 1 because `getBoundingClientRect` return decimal
                    if (top - pageTop <= dDistance + 1) {
                        if (isUndefined(nearestEl)) {
                            nearestEl = [href, top];
                        }
                        else if (top > nearestEl[1]) {
                            nearestEl = [href, top];
                        }
                    }
                }
                if (isArray(children)) {
                    reduceLinks(children);
                }
            });
        };
        reduceLinks(dList);
        const newHref = nearestEl ? nearestEl[0] : null;
        setActiveHref(newHref);
    }, [dDistance, dList, pageEl]);
    useIsomorphicLayoutEffect(() => {
        updateAnchor();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    useEffect(() => {
        if (pageEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(pageEl, 'scroll', { passive: true }).subscribe({
                next: () => {
                    updateAnchor();
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, pageEl, updateAnchor]);
    useEffect(() => {
        if (resizeEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.onResize(resizeEl, () => {
                updateAnchor();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, resizeEl, updateAnchor]);
    useEffect(() => {
        if (anchorRef.current && indicatorRef.current) {
            if (activeHref) {
                const el = anchorRef.current.querySelector(`.${dPrefix}anchor__link.is-active`);
                if (el) {
                    const rect = el.getBoundingClientRect();
                    const top = rect.top - anchorRef.current.getBoundingClientRect().top + rect.height / 2;
                    indicatorRef.current.style.cssText = `opacity:1;top:${top}px;`;
                }
            }
            else {
                indicatorRef.current.style.cssText += 'opacity:0;';
            }
        }
    });
    useImperativeHandle(ref, () => ({
        activeHref,
        updateAnchor,
    }), [activeHref, updateAnchor]);
    const handleLinkClick = (href) => {
        var _a, _b;
        if (!pageEl) {
            return;
        }
        const pageTop = pageEl.getBoundingClientRect().top;
        const scrollTop = pageEl.scrollTop;
        window.location.hash = `#${href}`;
        pageEl.scrollTop = scrollTop;
        const el = document.getElementById(href);
        if (el) {
            const top = el.getBoundingClientRect().top;
            const scrollTop = top - pageTop + pageEl.scrollTop - dDistance;
            (_b = (_a = dataRef.current).clearTid) === null || _b === void 0 ? void 0 : _b.call(_a);
            dataRef.current.clearTid = scrollTo(pageEl, {
                top: scrollTop,
                behavior: dScrollBehavior,
            });
        }
    };
    const linkNodes = (() => {
        const getNodes = (arr, level = 0) => arr.map((link) => {
            const { title: linkTitle, href: linkHref, target: linkTarget, children } = link;
            return (_jsxs(React.Fragment, { children: [_jsx("li", { children: _jsx("a", Object.assign({ className: getClassName(`${dPrefix}anchor__link`, {
                                'is-active': linkHref === activeHref,
                            }), style: { paddingLeft: 16 + level * 16 }, href: `#${linkHref}`, target: linkTarget, onClick: (e) => {
                                e.preventDefault();
                                onItemClick === null || onItemClick === void 0 ? void 0 : onItemClick(linkHref, link);
                                handleLinkClick(linkHref);
                            } }, { children: linkTitle !== null && linkTitle !== void 0 ? linkTitle : linkHref })) }), children && getNodes(children, level + 1)] }, `${linkHref}-${level}`));
        });
        return getNodes(dList);
    })();
    return (_jsxs("ul", Object.assign({}, restProps, { ref: anchorRef, className: getClassName(restProps.className, `${dPrefix}anchor`) }, { children: [_jsx("div", Object.assign({ className: `${dPrefix}anchor__indicator-track` }, { children: _jsx("div", Object.assign({ ref: indicatorRef, className: `${dPrefix}anchor__indicator-wrapper` }, { children: dIndicator === DOT_INDICATOR ? (_jsx("div", { className: `${dPrefix}anchor__dot-indicator` })) : dIndicator === LINE_INDICATOR ? (_jsx("div", { className: `${dPrefix}anchor__line-indicator` })) : (dIndicator) })) })), linkNodes] })));
}
export const DAnchor = React.forwardRef(Anchor);
DAnchor.DOT_INDICATOR = DOT_INDICATOR;
DAnchor.LINE_INDICATOR = LINE_INDICATOR;
//# sourceMappingURL=Anchor.js.map