import { Subject } from 'rxjs';
export interface ValidationErrors {
    [key: string]: any;
}
export declare type ValidatorFn = (control: AbstractControl) => ValidationErrors | null;
export declare type AsyncValidatorFn = (control: AbstractControl) => Promise<ValidationErrors | null>;
export declare type FormControlStatus = 'VALID' | 'INVALID' | 'PENDING' | 'DISABLED';
export declare abstract class AbstractControl {
    private _parent;
    private _pristine;
    private _errors;
    private _hasOwnPendingAsyncValidator;
    private _asyncValidationSubscription?;
    private _composedValidatorFn;
    private _composedAsyncValidatorFn;
    private _rawValidators;
    private _rawAsyncValidators;
    constructor(validators: ValidatorFn | ValidatorFn[] | null, asyncValidators: AsyncValidatorFn | AsyncValidatorFn[] | null);
    get validator(): ValidatorFn | null;
    set validator(validatorFn: ValidatorFn | null);
    get asyncValidator(): AsyncValidatorFn | null;
    set asyncValidator(asyncValidatorFn: AsyncValidatorFn | null);
    get parent(): FormGroup | null;
    get root(): AbstractControl;
    get value(): any;
    get status(): FormControlStatus;
    get errors(): ValidationErrors | null;
    get valid(): boolean;
    get invalid(): boolean;
    get pending(): boolean;
    get disabled(): boolean;
    get enabled(): boolean;
    get pristine(): boolean;
    get dirty(): boolean;
    readonly asyncVerifyComplete$: Subject<AbstractControl>;
    setValidators(validators: ValidatorFn | ValidatorFn[] | null): void;
    setAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[] | null): void;
    addValidators(validators: ValidatorFn | ValidatorFn[]): void;
    addAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[]): void;
    removeValidators(validators: ValidatorFn | ValidatorFn[]): void;
    removeAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[]): void;
    hasValidator(validator: ValidatorFn): boolean;
    hasAsyncValidator(validator: AsyncValidatorFn): boolean;
    clearValidators(): void;
    clearAsyncValidators(): void;
    markAsDirty(onlySelf?: boolean): void;
    markAsPristine(onlySelf?: boolean): void;
    markAsPending(onlySelf?: boolean): void;
    disable(onlySelf?: boolean): void;
    enable(onlySelf?: boolean): void;
    setParent(parent: FormGroup): void;
    setErrors(errors: ValidationErrors | null): void;
    updateValueAndValidity(onlySelf?: boolean): void;
    get(path: string[] | string): AbstractControl | null;
    getError(errorCode: string, path?: string[] | string): any;
    hasError(errorCode: string, path?: string[] | string): boolean;
    protected abstract _value: any;
    protected abstract _status: FormControlStatus;
    abstract setValue(value: any, onlySelf?: boolean): void;
    abstract patchValue(value: any, onlySelf?: boolean): void;
    abstract reset(value?: any, onlySelf?: boolean): void;
    protected abstract _updateValue(): void;
    protected abstract _forEachChild(cb: (c: AbstractControl) => void): void;
    protected abstract _anyControls(condition: (c: AbstractControl) => boolean): boolean;
    protected abstract _allControlsDisabled(): boolean;
    protected _isBoxedValue(formState: any): boolean;
    protected _anyControlsHaveStatus(status: FormControlStatus): boolean;
    protected _anyControlsDirty(): boolean;
    protected _parentMarkedDirty(onlySelf: boolean): boolean;
    protected _updatePristine(onlySelf?: boolean): void;
    protected _updateControlsErrors(): void;
    protected _calculateStatus(): FormControlStatus;
    protected _updateAncestors(onlySelf: boolean, skipPristineCheck: boolean): void;
    protected _runValidator(): ValidationErrors | null;
    protected _cancelExistingSubscription(): void;
    protected _runAsyncValidator(): void;
}
export declare class FormControl extends AbstractControl {
    protected _value: any;
    protected _status: FormControlStatus;
    constructor(formState?: any, validators?: ValidatorFn | ValidatorFn[] | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    setValue(value: any, onlySelf?: boolean): void;
    patchValue(value: any, onlySelf?: boolean): void;
    reset(formState?: any, onlySelf?: boolean): void;
    protected _updateValue(): void;
    protected _forEachChild(): void;
    protected _anyControls(): boolean;
    protected _allControlsDisabled(): boolean;
    private _applyFormState;
}
export declare class FormGroup extends AbstractControl {
    controls: {
        [key: string]: AbstractControl;
    };
    protected _value: any;
    protected _status: FormControlStatus;
    constructor(controls: {
        [key: string]: AbstractControl;
    }, validators?: ValidatorFn | ValidatorFn[] | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    addControl(name: string, control: AbstractControl): void;
    removeControl(name: string): void;
    setControl(name: string, control: AbstractControl): void;
    contains(controlName: string): boolean;
    setValue(value: {
        [key: string]: any;
    }, onlySelf?: boolean): void;
    patchValue(value: {
        [key: string]: any;
    }, onlySelf?: boolean): void;
    reset(value?: any, onlySelf?: boolean): void;
    protected _updateValue(): void;
    protected _forEachChild(cb: (v: any, k: string) => void): void;
    protected _anyControls(condition: (c: AbstractControl) => boolean): boolean;
    protected _allControlsDisabled(): boolean;
    private _setUpControls;
    private _reduceValue;
    private _reduceChildren;
    private _checkAllValuesPresent;
    private _throwIfControlMissing;
}
export declare class Validators {
    static min(min: number): ValidatorFn;
    static max(max: number): ValidatorFn;
    static required(control: AbstractControl): ValidationErrors | null;
    static email(control: AbstractControl): ValidationErrors | null;
    static minLength(minLength: number): ValidatorFn;
    static maxLength(maxLength: number): ValidatorFn;
    static pattern(pattern: string | RegExp): ValidatorFn;
}
