import { useEffect, useMemo, useState } from 'react';
import { useContextOptional } from '../../hooks';
import { DFormContext } from './Form';
export function useForm(initData) {
    const [form, setForm] = useState(() => initData());
    const [formChange, setFormChange] = useState(0);
    const updateForm = () => {
        setFormChange((n) => n + 1);
    };
    const initForm = () => {
        const data = initData();
        setForm(data);
        updateForm();
    };
    const formInstance = useMemo(() => ({
        form,
        initForm,
        updateForm,
    }), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [formChange]);
    return formInstance;
}
export function useFormControl(formControl) {
    const { gInstance: formInstance } = useContextOptional(DFormContext);
    const control = formControl === null || formControl === void 0 ? void 0 : formControl.control;
    useEffect(() => {
        if (control) {
            const ob = control.asyncVerifyComplete$.subscribe({
                next: (val) => {
                    if (val.dirty) {
                        formInstance === null || formInstance === void 0 ? void 0 : formInstance.updateForm();
                    }
                },
            });
            return () => {
                ob.unsubscribe();
            };
        }
    }, [control, formInstance]);
    return control
        ? [
            control.value,
            (val) => {
                if (control) {
                    control.markAsDirty(true);
                    control.setValue(val);
                    formInstance === null || formInstance === void 0 ? void 0 : formInstance.updateForm();
                }
            },
        ]
        : undefined;
}
//# sourceMappingURL=hooks.js.map