import { freeze } from 'immer';
import { useCallback, useEffect, useMemo, useState } from 'react';
import { useAsync, useMount } from '@react-devui/hooks';
import { useGridConfig } from '../../hooks';
export const MEDIA_QUERY_LIST = freeze(['xxl', 'xl', 'lg', 'md', 'sm', 'xs']);
function getMediaMatch(mqlList) {
    const mediaMatch = [];
    if (mqlList) {
        for (const [breakpoint, mql] of mqlList) {
            if (mql.matches) {
                mediaMatch.push(breakpoint);
            }
        }
    }
    return mediaMatch;
}
export function useMediaMatch() {
    const { dBreakpoints } = useGridConfig();
    const asyncCapture = useAsync();
    const mqlList = useMemo(() => {
        if (typeof window !== 'undefined') {
            return new Map(MEDIA_QUERY_LIST.map((breakpoint) => [breakpoint, window.matchMedia(`(min-width: ${dBreakpoints.get(breakpoint)}px)`)]));
        }
    }, [dBreakpoints]);
    const [mediaMatch, setMediaMatch] = useState(() => {
        if (typeof window !== 'undefined') {
            return getMediaMatch(mqlList);
        }
        return [];
    });
    const updateMediaMatch = useCallback(() => {
        const _mediaMatch = getMediaMatch(mqlList);
        setMediaMatch((draft) => {
            if (draft.length !== _mediaMatch.length) {
                return _mediaMatch;
            }
            else {
                for (const breakpoint of draft) {
                    if (!_mediaMatch.includes(breakpoint)) {
                        return _mediaMatch;
                    }
                }
                return draft;
            }
        });
    }, [mqlList]);
    useMount(() => {
        updateMediaMatch();
    });
    useEffect(() => {
        const [asyncGroup, asyncId] = asyncCapture.createGroup();
        asyncGroup.fromEvent(window, 'resize').subscribe({
            next: () => {
                updateMediaMatch();
            },
        });
        return () => {
            asyncCapture.deleteGroup(asyncId);
        };
    }, [asyncCapture, updateMediaMatch]);
    return mediaMatch;
}
//# sourceMappingURL=hooks.js.map