var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useCallback, useEffect, useId, useImperativeHandle, useRef, useState } from 'react';
import { useElement, useLockScroll } from '@react-devui/hooks';
import { getClassName, getPopupPosition } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate, handleModalKeyDown } from '../../utils';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { DPopoverFooter } from './PopoverFooter';
import { DPopoverHeader } from './PopoverHeader';
const TTANSITION_DURING = { enter: 86, leave: 100 };
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DPopover' });
function Popover(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dTrigger = 'hover', dContainer, dPlacement = 'top', dEscClosable = true, dArrow = true, dModal = false, dDisabled = false, dDistance = 10, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dZIndex, dHeader, dContent, dFooter, onVisibleChange, afterVisibleChange } = _a, restProps = __rest(_a, ["children", "dVisible", "dTrigger", "dContainer", "dPlacement", "dEscClosable", "dArrow", "dModal", "dDisabled", "dDistance", "dMouseEnterDelay", "dMouseLeaveDelay", "dZIndex", "dHeader", "dContent", "dFooter", "onVisibleChange", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const popoverRef = useRef(null);
    const popupRef = useRef(null);
    //#endregion
    const uniqueId = useId();
    const titleId = `${dPrefix}popover-title-${uniqueId}`;
    const bodyId = `${dPrefix}popover-content-${uniqueId}`;
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const isFixed = isUndefined(dContainer);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (isUndefined(dZIndex)) {
            if (isFixed) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return dZIndex;
        }
    })();
    const containerEl = useElement(isFixed
        ? () => {
            let el = document.getElementById(`${dPrefix}popover-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}popover-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a;
                const triggerEl = document.querySelector(`[data-popover-triggerid="${uniqueId}"]`);
                return (_a = triggerEl === null || triggerEl === void 0 ? void 0 : triggerEl.parentElement) !== null && _a !== void 0 ? _a : null;
            }
            : dContainer);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: -9999,
        left: -9999,
    });
    const [placement, setPlacement] = useState(dPlacement);
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useCallback(() => {
        const triggerEl = document.querySelector(`[data-popover-triggerid="${uniqueId}"]`);
        if (popupRef.current && triggerEl) {
            let currentPlacement = dPlacement;
            let space = [0, 0, 0, 0];
            if (!isFixed && containerEl) {
                const containerRect = containerEl.getBoundingClientRect();
                space = [
                    containerRect.top,
                    window.innerWidth - containerRect.left - containerRect.width,
                    window.innerHeight - containerRect.top - containerRect.height,
                    containerRect.left,
                ];
            }
            const position = getPopupPosition(popupRef.current, triggerEl, dPlacement, dDistance, space);
            if (position) {
                currentPlacement = position.placement;
                setPlacement(position.placement);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            else {
                const position = getPopupPosition(popupRef.current, triggerEl, placement, dDistance);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            let transformOrigin = 'center bottom';
            switch (currentPlacement) {
                case 'top':
                    transformOrigin = 'center bottom';
                    break;
                case 'top-left':
                    transformOrigin = '20px bottom';
                    break;
                case 'top-right':
                    transformOrigin = 'calc(100% - 20px) bottom';
                    break;
                case 'right':
                    transformOrigin = 'left center';
                    break;
                case 'right-top':
                    transformOrigin = 'left 12px';
                    break;
                case 'right-bottom':
                    transformOrigin = 'left calc(100% - 12px)';
                    break;
                case 'bottom':
                    transformOrigin = 'center top';
                    break;
                case 'bottom-left':
                    transformOrigin = '20px top';
                    break;
                case 'bottom-right':
                    transformOrigin = 'calc(100% - 20px) top';
                    break;
                case 'left':
                    transformOrigin = 'right center';
                    break;
                case 'left-top':
                    transformOrigin = 'right 12px';
                    break;
                case 'left-bottom':
                    transformOrigin = 'right calc(100% - 12px)';
                    break;
                default:
                    break;
            }
            setTransformOrigin(transformOrigin);
        }
    }, [containerEl, dDistance, dPlacement, isFixed, placement, uniqueId]);
    useLockScroll(dModal && visible);
    const prevActiveEl = useRef(null);
    useEffect(() => {
        if (dModal) {
            if (visible) {
                prevActiveEl.current = document.activeElement;
                if (popoverRef.current) {
                    popoverRef.current.focus({ preventScroll: true });
                }
            }
            else if (prevActiveEl.current) {
                prevActiveEl.current.focus({ preventScroll: true });
            }
        }
    }, [dModal, visible]);
    const headerNode = (() => {
        if (dHeader) {
            const node = isString(dHeader) ? _jsx(DPopoverHeader, { children: dHeader }) : dHeader;
            return React.cloneElement(node, Object.assign(Object.assign({}, node.props), { __id: titleId, __onClose: () => {
                    changeVisible(false);
                } }));
        }
    })();
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    return (_jsx(DPopup, Object.assign({ dPopup: (_a) => {
            var { pOnClick, pOnMouseEnter, pOnMouseLeave } = _a, restPProps = __rest(_a, ["pOnClick", "pOnMouseEnter", "pOnMouseLeave"]);
            return (_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING, onEnterRendered: updatePosition, afterEnter: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                }, afterLeave: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                } }, { children: (state) => {
                    var _a, _b, _c, _d;
                    let transitionStyle = {};
                    switch (state) {
                        case 'enter':
                            transitionStyle = { transform: 'scale(0.3)', opacity: 0 };
                            break;
                        case 'entering':
                            transitionStyle = {
                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.enter}ms ease-out`).join(', '),
                                transformOrigin,
                            };
                            break;
                        case 'leaving':
                            transitionStyle = {
                                transform: 'scale(0.3)',
                                opacity: 0,
                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.leave}ms ease-in`).join(', '),
                                transformOrigin,
                            };
                            break;
                        default:
                            break;
                    }
                    return (_jsxs("div", Object.assign({}, restProps, { ref: popoverRef, className: getClassName(restProps.className, `${dPrefix}popover`, `${dPrefix}popover--` + placement), style: Object.assign(Object.assign({}, restProps.style), { display: state === 'leaved' ? 'none' : undefined, zIndex }), tabIndex: -1, role: (_a = restProps.role) !== null && _a !== void 0 ? _a : (dModal ? 'alertdialog' : 'alert'), "aria-modal": (_b = restProps['aria-modal']) !== null && _b !== void 0 ? _b : dModal, "aria-labelledby": (_c = restProps['aria-labelledby']) !== null && _c !== void 0 ? _c : (headerNode ? titleId : undefined), "aria-describedby": (_d = restProps['aria-describedby']) !== null && _d !== void 0 ? _d : bodyId, onKeyDown: (e) => {
                            var _a;
                            (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                            if (dEscClosable && e.code === 'Escape') {
                                changeVisible(false);
                            }
                            if (dModal) {
                                handleModalKeyDown(e);
                            }
                        } }, { children: [dModal && (_jsx("div", { className: `${dPrefix}popover__mask`, onClick: () => {
                                    changeVisible(false);
                                } })), _jsxs("div", Object.assign({}, restPProps, { ref: popupRef, className: `${dPrefix}popover__content`, style: Object.assign(Object.assign({}, popupPositionStyle), transitionStyle), onClick: (e) => {
                                    var _a;
                                    (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                    pOnClick === null || pOnClick === void 0 ? void 0 : pOnClick(e);
                                }, onMouseEnter: (e) => {
                                    var _a;
                                    (_a = restProps.onMouseEnter) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                    pOnMouseEnter === null || pOnMouseEnter === void 0 ? void 0 : pOnMouseEnter(e);
                                }, onMouseLeave: (e) => {
                                    var _a;
                                    (_a = restProps.onMouseLeave) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                    pOnMouseLeave === null || pOnMouseLeave === void 0 ? void 0 : pOnMouseLeave(e);
                                } }, { children: [dArrow && _jsx("div", { className: `${dPrefix}popover__arrow` }), headerNode, _jsx("div", Object.assign({ id: bodyId, className: `${dPrefix}popover__body` }, { children: dContent })), dFooter &&
                                        React.cloneElement(dFooter, Object.assign(Object.assign({}, dFooter.props), { __onClose: () => {
                                                changeVisible(false);
                                            } }))] }))] })));
                } })));
        }, dVisible: visible, dContainer: containerEl, dTrigger: dTrigger, dDisabled: dDisabled, dEscClosable: dEscClosable, dMouseEnterDelay: dMouseEnterDelay, dMouseLeaveDelay: dMouseLeaveDelay, dUpdatePosition: updatePosition, onVisibleChange: changeVisible }, { children: (_a) => {
            var { pOnClick, pOnMouseEnter, pOnMouseLeave } = _a, restPProps = __rest(_a, ["pOnClick", "pOnMouseEnter", "pOnMouseLeave"]);
            return React.cloneElement(children, Object.assign(Object.assign(Object.assign({}, children.props), restPProps), { 'data-popover-triggerid': uniqueId, onClick: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onClick) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    pOnClick === null || pOnClick === void 0 ? void 0 : pOnClick(e);
                }, onMouseEnter: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onMouseEnter) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    pOnMouseEnter === null || pOnMouseEnter === void 0 ? void 0 : pOnMouseEnter(e);
                }, onMouseLeave: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onMouseLeave) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    pOnMouseLeave === null || pOnMouseLeave === void 0 ? void 0 : pOnMouseLeave(e);
                } }));
        } })));
}
export const DPopover = React.forwardRef(Popover);
DPopover.Header = DPopoverHeader;
DPopover.Footer = DPopoverFooter;
//# sourceMappingURL=Popover.js.map