import { jsx as _jsx } from "react/jsx-runtime";
import { isBoolean, isNumber, isUndefined, nth } from 'lodash';
import React, { useCallback, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import { useAsync, useForceUpdate } from '@react-devui/hooks';
import { toPx } from '@react-devui/utils';
import { useComponentConfig } from '../../hooks';
import { registerComponentMate } from '../../utils';
const EMPTY = Symbol();
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DVirtualScroll' });
function VirtualScroll(props, ref) {
    const { children, dList, dFillNode, dItemRender, dItemSize, dItemNested, dItemKey, dFocusable = true, dFocusItem, dSize, dPadding, dHorizontal = false, dEmptyRender, dExpands, onScrollEnd, } = useComponentConfig(COMPONENT_NAME, props);
    const dataRef = useRef({
        listCache: new Map(),
    });
    const asyncCapture = useAsync();
    const forceUpdate = useForceUpdate();
    const [scrollEl, setScrollEl] = useState(null);
    const scrollRef = useCallback((el) => {
        setScrollEl(el);
    }, []);
    const [scrollPosition, setScrollPosition] = useState(0);
    const getItemSize = useMemo(() => (isNumber(dItemSize) ? () => dItemSize : dItemSize), [dItemSize]);
    const checkFocusable = useMemo(() => (isBoolean(dFocusable) ? () => dFocusable : dFocusable), [dFocusable]);
    const [itemsMap, totalSize, firstFocusableItem, lastFocusableItem] = useMemo(() => {
        let accSize = 0;
        let firstFocusableItem;
        let lastFocusableItem;
        const items = new Map();
        const reduceArr = (arr) => {
            var _a;
            let size = 0;
            for (const item of arr) {
                if (checkFocusable(item)) {
                    lastFocusableItem = item;
                    if (isUndefined(firstFocusableItem)) {
                        firstFocusableItem = item;
                    }
                }
                size += getItemSize(item);
                accSize += getItemSize(item);
                const data = { item, accSize, nestedSize: 0 };
                items.set(dItemKey(item), data);
                const nestedData = dItemNested === null || dItemNested === void 0 ? void 0 : dItemNested(item);
                if (nestedData && nestedData.list && (isUndefined(dExpands) || dExpands.has(dItemKey(item)))) {
                    if (nestedData.list.length === 0) {
                        data.nestedSize = (_a = nestedData.emptySize) !== null && _a !== void 0 ? _a : 0;
                        size += data.nestedSize;
                        accSize += data.nestedSize;
                    }
                    else {
                        data.nestedSize = reduceArr(nestedData.list);
                        size += data.nestedSize;
                    }
                }
            }
            return size;
        };
        return [items, reduceArr(dList), firstFocusableItem, lastFocusableItem];
    }, [checkFocusable, dExpands, dItemKey, dItemNested, dList, getItemSize]);
    let elSize;
    let elPaddingSize;
    if (scrollEl) {
        elSize = scrollEl[dHorizontal ? 'clientWidth' : 'clientHeight'];
        elPaddingSize = toPx(getComputedStyle(scrollEl).getPropertyValue(dHorizontal ? 'padding-left' : 'padding-top'), true);
    }
    const ulSize = dSize !== null && dSize !== void 0 ? dSize : elSize;
    const paddingSize = dPadding !== null && dPadding !== void 0 ? dPadding : elPaddingSize;
    useEffect(() => {
        if (isUndefined(dSize)) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            if (scrollEl) {
                asyncGroup.onResize(scrollEl, () => {
                    forceUpdate();
                });
            }
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, dSize, forceUpdate, scrollEl]);
    const list = (() => {
        if (isUndefined(ulSize) || isUndefined(paddingSize)) {
            return [];
        }
        const maxScrollSize = Math.max(totalSize + paddingSize * 2 - ulSize, 0);
        const _scrollPosition = Math.min(scrollPosition, maxScrollSize);
        let totalAccSize = 0;
        const startSize = _scrollPosition - ulSize - paddingSize;
        const endSize = _scrollPosition + ulSize + ulSize - paddingSize;
        let hasStart = false;
        let hasEnd = false;
        const getList = (arr, parent = []) => {
            var _a, _b, _c, _d;
            const fillSize = [0, 0];
            const list = [];
            const setsize = arr.filter((item) => { var _a, _b; return item !== EMPTY && ((_b = (_a = dItemNested === null || dItemNested === void 0 ? void 0 : dItemNested(item)) === null || _a === void 0 ? void 0 : _a.asItem) !== null && _b !== void 0 ? _b : true); }).length;
            for (const [index, item] of arr.entries()) {
                let key = '';
                let size = 0;
                let nestedList;
                let childrenSize = 0;
                let emptyNode;
                if (item === EMPTY) {
                    size = (_b = (_a = dItemNested === null || dItemNested === void 0 ? void 0 : dItemNested(parent[parent.length - 1])) === null || _a === void 0 ? void 0 : _a.emptySize) !== null && _b !== void 0 ? _b : 0;
                    emptyNode = _jsx(React.Fragment, { children: dEmptyRender === null || dEmptyRender === void 0 ? void 0 : dEmptyRender(parent[parent.length - 1]) }, "$$empty");
                }
                else {
                    key = dItemKey(item);
                    size = getItemSize(item);
                    nestedList = (_c = dItemNested === null || dItemNested === void 0 ? void 0 : dItemNested(item)) === null || _c === void 0 ? void 0 : _c.list;
                    if (nestedList) {
                        childrenSize = itemsMap.get(key).nestedSize;
                    }
                }
                if (hasEnd) {
                    fillSize[1] += size + childrenSize;
                    continue;
                }
                totalAccSize += size;
                if (nestedList) {
                    if (totalAccSize + childrenSize > startSize) {
                        let childrenList = [];
                        if (isUndefined(dExpands)) {
                            childrenList = getList(nestedList.length === 0 ? [EMPTY] : nestedList, parent.concat([item]));
                        }
                        else {
                            childrenList = (_d = dataRef.current.listCache.get(key)) !== null && _d !== void 0 ? _d : [];
                            if (dExpands.has(dItemKey(item))) {
                                childrenList = getList(nestedList.length === 0 ? [EMPTY] : nestedList, parent.concat([item]));
                                dataRef.current.listCache.set(key, childrenList);
                            }
                        }
                        list.push(dItemRender(item, index, {
                            iARIA: { 'aria-level': parent.length + 1, 'aria-setsize': setsize, 'aria-posinset': index + 1 },
                            iChildren: childrenList,
                        }, parent));
                    }
                    else {
                        totalAccSize += childrenSize;
                        fillSize[0] += size + childrenSize;
                    }
                }
                else if (!hasStart) {
                    if (totalAccSize > startSize) {
                        list.push(item === EMPTY
                            ? emptyNode
                            : dItemRender(item, index, {
                                iARIA: { 'aria-level': parent.length + 1, 'aria-setsize': setsize, 'aria-posinset': index + 1 },
                            }, parent));
                        hasStart = true;
                    }
                    else {
                        fillSize[0] += size;
                    }
                }
                else if (!hasEnd) {
                    if (totalAccSize > endSize) {
                        hasEnd = true;
                        fillSize[1] += size;
                    }
                    else {
                        list.push(item === EMPTY
                            ? emptyNode
                            : dItemRender(item, index, {
                                iARIA: { 'aria-level': parent.length + 1, 'aria-setsize': setsize, 'aria-posinset': index + 1 },
                            }, parent));
                    }
                }
            }
            if (fillSize[0] > 0) {
                list.unshift(React.cloneElement(dFillNode, Object.assign(Object.assign({}, dFillNode.props), { key: '$$fill-size-0', style: Object.assign(Object.assign({}, dFillNode.props.style), { overflow: 'hidden', [dHorizontal ? 'height' : 'width']: 0, [dHorizontal ? 'width' : 'height']: fillSize[0] }), 'aria-hidden': true })));
            }
            if (fillSize[1] > 0) {
                list.push(React.cloneElement(dFillNode, Object.assign(Object.assign({}, dFillNode.props), { key: '$$fill-size-1', style: Object.assign(Object.assign({}, dFillNode.props.style), { overflow: 'hidden', [dHorizontal ? 'height' : 'width']: 0, [dHorizontal ? 'width' : 'height']: fillSize[1] }), 'aria-hidden': true })));
            }
            return list;
        };
        return getList(dList);
    })();
    const scrollTo = useCallback((num) => {
        if (scrollEl) {
            scrollEl[dHorizontal ? 'scrollLeft' : 'scrollTop'] = num;
        }
    }, [dHorizontal, scrollEl]);
    const scrollToItem = useCallback((item) => {
        if (!isUndefined(paddingSize)) {
            const findItem = itemsMap.get(dItemKey(item));
            if (!isUndefined(findItem)) {
                scrollTo(findItem.accSize - getItemSize(findItem.item) + paddingSize);
            }
        }
    }, [dItemKey, getItemSize, itemsMap, paddingSize, scrollTo]);
    const scrollToStart = useCallback(() => {
        scrollTo(0);
        return firstFocusableItem;
    }, [firstFocusableItem, scrollTo]);
    const scrollToEnd = useCallback(() => {
        if (scrollEl) {
            scrollTo(scrollEl[dHorizontal ? 'scrollWidth' : 'scrollHeight']);
        }
        return lastFocusableItem;
    }, [dHorizontal, lastFocusableItem, scrollEl, scrollTo]);
    const scrollByStep = useCallback((step) => {
        if (!isUndefined(paddingSize)) {
            if (isUndefined(dFocusItem)) {
                return step > 0 ? scrollToStart() : scrollToEnd();
            }
            let findItem;
            let offsetSize;
            if (scrollEl) {
                let index = -1;
                let findIndex = -1;
                const accSizeList = [];
                for (const iterator of itemsMap) {
                    index += 1;
                    if (dItemKey(iterator[1].item) === dItemKey(dFocusItem)) {
                        findIndex = index;
                    }
                    accSizeList.push(iterator[1]);
                }
                if (findIndex !== -1) {
                    if (step < 0) {
                        for (let index = findIndex - 1, n = 0; n < accSizeList.length; index--, n++) {
                            const accSizeItem = nth(accSizeList, index);
                            if (accSizeItem && checkFocusable(accSizeItem.item)) {
                                findItem = accSizeItem.item;
                                offsetSize = [accSizeItem.accSize - getItemSize(findItem) + paddingSize, accSizeItem.accSize + paddingSize];
                                break;
                            }
                        }
                    }
                    else {
                        for (let index = findIndex + 1, n = 0; n < accSizeList.length; index++, n++) {
                            const accSizeItem = nth(accSizeList, index % accSizeList.length);
                            if (accSizeItem && checkFocusable(accSizeItem.item)) {
                                findItem = accSizeItem.item;
                                offsetSize = [accSizeItem.accSize - getItemSize(findItem) + paddingSize, accSizeItem.accSize + paddingSize];
                                break;
                            }
                        }
                    }
                }
                if (!isUndefined(offsetSize)) {
                    const listElScrollPosition = scrollEl[dHorizontal ? 'scrollLeft' : 'scrollTop'];
                    const listElClientSize = scrollEl[dHorizontal ? 'clientWidth' : 'clientHeight'];
                    if (listElScrollPosition > offsetSize[1]) {
                        scrollTo(offsetSize[0] - paddingSize);
                    }
                    else if (offsetSize[0] > listElScrollPosition + listElClientSize) {
                        scrollTo(offsetSize[1] - listElClientSize + paddingSize);
                    }
                    else {
                        if (step > 0) {
                            if (offsetSize[1] > listElScrollPosition + listElClientSize) {
                                scrollTo(offsetSize[1] - listElClientSize + paddingSize);
                            }
                        }
                        else {
                            if (listElScrollPosition > offsetSize[0]) {
                                scrollTo(offsetSize[0] - paddingSize);
                            }
                        }
                    }
                }
            }
            return findItem;
        }
    }, [checkFocusable, dFocusItem, dHorizontal, dItemKey, getItemSize, itemsMap, paddingSize, scrollEl, scrollTo, scrollToEnd, scrollToStart]);
    useImperativeHandle(ref, () => ({
        scrollToItem,
        scrollByStep,
        scrollToStart,
        scrollToEnd,
    }), [scrollByStep, scrollToEnd, scrollToStart, scrollToItem]);
    return children({
        vsScrollRef: scrollRef,
        vsRender: list,
        vsOnScroll: (e) => {
            setScrollPosition(e.currentTarget[dHorizontal ? 'scrollLeft' : 'scrollTop']);
            if (dHorizontal
                ? e.currentTarget.scrollLeft + e.currentTarget.clientWidth === e.currentTarget.scrollWidth
                : e.currentTarget.scrollTop + e.currentTarget.clientHeight === e.currentTarget.scrollHeight) {
                onScrollEnd === null || onScrollEnd === void 0 ? void 0 : onScrollEnd();
            }
        },
    });
}
export const DVirtualScroll = React.forwardRef(VirtualScroll);
//# sourceMappingURL=VirtualScroll.js.map